c *****************************************
      subroutine xsismhri(ear,ne,param,Ifl,photar)
c *****************************************
c *****************************************
c     number of model parameters: 14
c     1: Te, electron temperature in keV
c        (if Te > 0 exact redistribution function (RF),
c        if Te < 0 isotropic approximation for RF)
c     2: Tbb, blackbody temperature at in the inner edge in keV
c        (if < 0 then only black body component)
c     3: tau, vertical optical depth of the corona
c        if tau > 0 isotropic source function (SF) starting
c                   from the 3rd scattering
c        if tau < 0 isotropic SF starting from the 1st scattering
c     4: cosIncl, cosine of inclination angle
c     5: Rin/Rc, inner radius of the disk in units of the corona 
c     6, R, amount of reflection (if < 0 then only reflection component)
c     7: FeAb, iron abundance in units of solar
c     8: MeAb, abundance of heavy elements in units of solar
c     9: xi, disk ionization parameter L/(nR^2)
c    10: temp, disk temperature for reflection in K
c    11: beta, reflection emissivity law (r^beta)IF=-10 then
c         non-rotating disk, IF=10 then  (1.-sqrt(6./rg))/rg**3
c    12: Rin/Rg, inner radius of the disk (Schwarzschild units)
c    13: Rout/Rg, outer radius of the disk 
c    14: redshift
c
c *****************************************
c     algorithm:
c       exact solution of the radiative transfer equation
c       in a SPHERICAL CORONA
c       neglecting the energy balance
c       geometry: cold disk with the hole in the center
c                 (sombrero model)
c       electrons: Maxwellian 
c       includes: Comptonization of the BB radiation from the 
c       disk, and relativistic (like in diskline) reflection; 
c 
c *****************************************
      implicit none
      integer np,i,nem
      PARAMETER(nem=5000) 
      PARAMETER(np=20)
      integer ne,ne1,iigeom,iidistf,iire,Ifl 
      real*4  param(*),ear(0:ne),photar(ne)
      real*4  pa0(np)
      real*8  pac(np)    
      real*8  phcon(nem),phref(nem),phblb(nem),obj(nem)  
      real*4  phtot(nem)
      logical firstcall
      save firstcall
      data firstcall/.true./
c      logical fl0
      data    pa0/np*9999./

      ne1=ne+1 
c check if the parameters were changed  
c      do n=1,np
c         if(param(n).ne.pa0(n)) fl0=.true.
c      enddo
c electron temperature
         pac(1)=dble(param(1))
c power-law index
         pac(2)=0d0
c gamma_min
         pac(3)=0d0
c gamma_max
         pac(4)=0d0
c disc temperature (keV)
         pac(5)=dble(param(2))
c Thomson optical depth
         pac(6)=dble(param(3))
c cosine of inclination angle
         pac(7)=dble(param(4))
c g, or Rin/Rc (inner radius of the disc)
         pac(8)=dble(param(5))
c normalization of reflection
         pac(9)=dabs(dble(param(6)))
c iron abundance
         pac(10)=dble(param(7))
c "metal" abundances
         pac(11)=dble(param(8))
c xi (ionization parameter)
         pac(12)=dble(param(9))
c disc temp (for ionization)
         pac(13)=dble(param(10))
c redshift
         pac(14)=dble(param(14))
c for relativistic disk reflection 
c beta, reflection emissivity law (r^beta)
         pac(15)=dble(param(11))
c Rin/Rg, inner radius of the disk
         pac(16)=dble(param(12))
c Rout/Rg, outer radius of the disk 
         pac(17)=dble(param(13))
         pac(18)=0d0
         pac(19)=0d0
         pac(20)=0d0

C IGEOM=0, 3 SLAB geometry
C IGEOM=1, 4 CYLINDER geometry (with T0TAUG=H/R)
C IGEOM=2, 5 HEMISPHERE geometry  (TOTAUG=1, H=R)
c for IIGEOM.gt.2 -- holly disk 
c HEMISPHERE 
      IIGEOM=5
C IDISTF=0 Maxwellian electron distribution 
C IDISTF=1 monoenergetic electron distribution 
C IDISTF=2 power-law  electron distribution 
C IDISTF=4 Maxwellian + power-law tail
      IIDISTF=0
c IIRE=0 neutral reflector (H and He are always ionized) & static 
c IIRE=1 ionized reflector & static 
c IIRE=2 neutral reflector & relativistic 
c IIRE=3 ionized reflector & relativistic
      IIRE=3

c object's frame energies (in mc2) 
         do i=0,ne
            obj(i+1)=dble((1.+param(14))*ear(i)/511.) 
         enddo 
c ---- 
        call ismco(pac,obj,phcon,phref,phblb,ne1,iigeom,iidistf,iire) 
c ----
c keep the new parameters  
c         do n=1,np
c            pa0(n)=param(n)
c         enddo
c         fl0=.false.
      do i=1,ne1
      if(param(6).ge.0..and.param(2).ge.0.) then
c       phtot(i)=sngl(phcon(i)+phref(i)+phblb(i))
       phtot(i)=sngl(phcon(i)+phref(i))
      endif
      if(param(6).lt.0.) phtot(i)=sngl(phref(i))
      if(param(2).lt.0.) phtot(i)=sngl(phblb(i))
      if(param(2).lt.0..and.param(6).lt.0.)
     + phtot(i)=sngl(phblb(i)+phref(i))
      enddo
c integrate over the bin size
      do i=1,ne
        photar(i)=(phtot(i+1)+phtot(i))*(ear(i)-ear(i-1))/2.
      enddo


      return
      end
